@@ -17,12 +17,11 @@ from aggregates import *
 #       people.group_by('country','city').pivot_by('gender').annotate(Avg('age'))
 
 # TODO: [0, None, NA] -- they all mean the same. Need precise tests which should be used when.
-#       Also in some cases there are no columns at all -- this is simply unacceptable.
 
 # TODO: see cast() code for more TODOs :)
 
 __doc__ = """
->>> from db import Dataset, in_, not_
+>>> from db import Dataset
 >>> from aggregates import *
 >>> import yaml
 >>> data = yaml.load(open('example_data/people.yaml'))
@@ -37,7 +36,7 @@ __doc__ = """
  +------------+
  | Count(all) |
  +------------+
- |         14 |
+ |         18 |
  +------------+
 
 # count, implicit aggregate
@@ -46,8 +45,8 @@ __doc__ = """
  +--------+------------+
  | gender | Count(all) |
  +--------+------------+
- | female |          2 |
- |   male |         12 |
+ | female |          3 |
+ |   male |         13 |
  +--------+------------+
 
 # count, explicit aggregate
@@ -56,8 +55,8 @@ __doc__ = """
  +--------+------------+
  | gender | Count(all) |
  +--------+------------+
- | female |          2 |
- |   male |         12 |
+ | female |          3 |
+ |   male |         13 |
  +--------+------------+
 
 >>> cast_cons(q, ['gender'], [], Count('nick'))
@@ -79,91 +78,74 @@ __doc__ = """
  +--------+------+------------+
  | female | male | Count(all) |
  +--------+------+------------+
- |      2 |   12 |         14 |
+ |      3 |   13 |         18 |
  +--------+------+------------+
 
->>> cast_cons(people.all(), [], ['country'], Count('nick'))
- +---------+---------+-------------+-------------+--------+--------+-------------+-----+-------------+
- | England | Finland | Netherlands | New Zealand | Norway | Sweden | Switzerland | USA | Count(nick) |
- +---------+---------+-------------+-------------+--------+--------+-------------+-----+-------------+
- |     N/A |     N/A |         N/A |         N/A |    N/A |      1 |         N/A |   3 |           4 |
- +---------+---------+-------------+-------------+--------+--------+-------------+-----+-------------+
+>>> cast_cons(people.all(), [], ['born__country'], Count('nick'))
+ +---------+---------+-------------+-------------+--------+--------------+--------+-------------+-----+-------------+
+ | England | Finland | Netherlands | New Zealand | Norway | South Africa | Sweden | Switzerland | USA | Count(nick) |
+ +---------+---------+-------------+-------------+--------+--------------+--------+-------------+-----+-------------+
+ |     N/A |     N/A |         N/A |         N/A |    N/A |          N/A |      1 |         N/A |   3 |           4 |
+ +---------+---------+-------------+-------------+--------+--------------+--------+-------------+-----+-------------+
 
 # count, implicit aggregate
 
->>> cast_cons(q, ['country'])
- +-------------+------------+
- |     country | Count(all) |
- +-------------+------------+
- |     England |          2 |
- |     Finland |          1 |
- | Netherlands |          1 |
- | New Zealand |          1 |
- |      Norway |          1 |
- |      Sweden |          1 |
- | Switzerland |          1 |
- |         USA |          6 |
- +-------------+------------+
+>>> cast_cons(q, ['born__country'])
+ +---------------+------------+
+ | born__country | Count(all) |
+ +---------------+------------+
+ |       England |          2 |
+ |       Finland |          1 |
+ |   Netherlands |          2 |
+ |   New Zealand |          2 |
+ |        Norway |          1 |
+ |  South Africa |          1 |
+ |        Sweden |          1 |
+ |   Switzerland |          1 |
+ |           USA |          7 |
+ +---------------+------------+
 
 # count, explicit aggregate
 
->>> cast_cons(q, ['country'], [], Count())
- +-------------+------------+
- |     country | Count(all) |
- +-------------+------------+
- |     England |          2 |
- |     Finland |          1 |
- | Netherlands |          1 |
- | New Zealand |          1 |
- |      Norway |          1 |
- |      Sweden |          1 |
- | Switzerland |          1 |
- |         USA |          6 |
- +-------------+------------+
+>>> cast_cons(q, ['born__country'], [], Count())
+ +---------------+------------+
+ | born__country | Count(all) |
+ +---------------+------------+
+ |       England |          2 |
+ |       Finland |          1 |
+ |   Netherlands |          2 |
+ |   New Zealand |          2 |
+ |        Norway |          1 |
+ |  South Africa |          1 |
+ |        Sweden |          1 |
+ |   Switzerland |          1 |
+ |           USA |          7 |
+ +---------------+------------+
 
 # count, multiple factors (two)
 
->>> cast_cons(q, ['country','city'], [], Count())
- +-------------+---------------------+------------+
- |     country |                city | Count(all) |
- +-------------+---------------------+------------+
- |     England |    Great Torrington |          1 |
- |     England |  Maida Vale, London |          1 |
- |     Finland |            Helsinki |          1 |
- | Netherlands |           Rotterdam |          1 |
- | New Zealand |            Auckland |          1 |
- |      Norway |                Oslo |          1 |
- |      Sweden |           Stockholm |          1 |
- | Switzerland |          Winterthur |          1 |
- |         USA | Betty Jean Jennings |          1 |
- |         USA |             Chicago |          1 |
- |         USA |           Milwaukee |          1 |
- |         USA |            New York |          1 |
- |         USA |                None |          1 |
- |         USA |            San Jose |          1 |
- +-------------+---------------------+------------+
-
-# count, multiple factors (three)
-
->>> cast_cons(q, ['country','city','gender'], [], Count())
- +-------------+---------------------+--------+------------+
- |     country |                city | gender | Count(all) |
- +-------------+---------------------+--------+------------+
- |     England |    Great Torrington |   male |          1 |
- |     England |  Maida Vale, London |   male |          1 |
- |     Finland |            Helsinki |   male |          1 |
- | Netherlands |           Rotterdam |   male |          1 |
- | New Zealand |            Auckland |   male |          1 |
- |      Norway |                Oslo |   male |          1 |
- |      Sweden |           Stockholm |   male |          1 |
- | Switzerland |          Winterthur |   male |          1 |
- |         USA | Betty Jean Jennings | female |          1 |
- |         USA |             Chicago |   male |          1 |
- |         USA |           Milwaukee |   male |          1 |
- |         USA |            New York |   male |          1 |
- |         USA |                None | female |          1 |
- |         USA |            San Jose |   male |          1 |
- +-------------+---------------------+--------+------------+
+>>> cast_cons(q, ['born__country','born__city'], [], Count())
+ +---------------+---------------------+------------+
+ | born__country |          born__city | Count(all) |
+ +---------------+---------------------+------------+
+ |       England |    Great Torrington |          1 |
+ |       England |  Maida Vale, London |          1 |
+ |       Finland |            Helsinki |          1 |
+ |   Netherlands |           Amsterdam |          1 |
+ |   Netherlands |           Rotterdam |          1 |
+ |   New Zealand |            Auckland |          2 |
+ |        Norway |                Oslo |          1 |
+ |  South Africa |            Pretoria |          1 |
+ |        Sweden |           Stockholm |          1 |
+ |   Switzerland |          Winterthur |          1 |
+ |           USA |                None |          1 |
+ |           USA | Betty Jean Jennings |          1 |
+ |           USA |             Chicago |          1 |
+ |           USA |           Milwaukee |          1 |
+ |           USA |            New York |          1 |
+ |           USA |            San Jose |          1 |
+ |           USA |             Seattle |          1 |
+ +---------------+---------------------+------------+
 
 # explicit custom aggregate, no grouper factor
 
@@ -171,164 +153,143 @@ __doc__ = """
  +----------+
  | Avg(age) |
  +----------+
- |     86.8 |
+ |     79.5 |
  +----------+
 
 >>> cast_cons(q, [], [], Median('age'))
  +-------------+
  | Median(age) |
  +-------------+
- |          79 |
+ |        73.5 |
  +-------------+
 
 # aggregate by one key, Avg
 
->>> cast_cons(q, ['country'], [], Avg('age'))
- +-------------+----------+
- |     country | Avg(age) |
- +-------------+----------+
- |     England |    164.5 |
- |     Finland |     40.0 |
- | Netherlands |     79.0 |
- | New Zealand |      N/A |
- |      Norway |     83.0 |
- |      Sweden |    102.0 |
- | Switzerland |     75.0 |
- |         USA |     70.0 |
- +-------------+----------+
+>>> cast_cons(q, ['born__country'], [], Avg('age'))
+ +---------------+----------+
+ | born__country | Avg(age) |
+ +---------------+----------+
+ |       England |    164.5 |
+ |       Finland |     40.0 |
+ |   Netherlands |     64.0 |
+ |   New Zealand |      N/A |
+ |        Norway |     83.0 |
+ |  South Africa |     41.0 |
+ |        Sweden |    102.0 |
+ |   Switzerland |     75.0 |
+ |           USA |     67.7 |
+ +---------------+----------+
 
 # aggregate by one key, Max
 
->>> cast_cons(q, ['country'], [], Max('age'))
- +-------------+----------+
- |     country | Max(age) |
- +-------------+----------+
- |     England |      232 |
- |     Finland |       40 |
- | Netherlands |       79 |
- | New Zealand |      N/A |
- |      Norway |       83 |
- |      Sweden |      102 |
- | Switzerland |       75 |
- |         USA |       88 |
- +-------------+----------+
+>>> cast_cons(q, ['born__country'], [], Max('age'))
+ +---------------+----------+
+ | born__country | Max(age) |
+ +---------------+----------+
+ |       England |      232 |
+ |       Finland |       40 |
+ |   Netherlands |       79 |
+ |   New Zealand |      N/A |
+ |        Norway |       83 |
+ |  South Africa |       41 |
+ |        Sweden |      102 |
+ |   Switzerland |       75 |
+ |           USA |       88 |
+ +---------------+----------+
 
-# aggregate by multiple keys (two)
+# aggregate by multiple keys
 
->>> cast_cons(q, ['country','city'], [], Avg('age'))
- +-------------+---------------------+----------+
- |     country |                city | Avg(age) |
- +-------------+---------------------+----------+
- |     England |    Great Torrington |    232.0 |
- |     England |  Maida Vale, London |     97.0 |
- |     Finland |            Helsinki |     40.0 |
- | Netherlands |           Rotterdam |     79.0 |
- | New Zealand |            Auckland |      N/A |
- |      Norway |                Oslo |     83.0 |
- |      Sweden |           Stockholm |    102.0 |
- | Switzerland |          Winterthur |     75.0 |
- |         USA | Betty Jean Jennings |     85.0 |
- |         USA |             Chicago |     60.0 |
- |         USA |           Milwaukee |     72.0 |
- |         USA |            New York |     56.0 |
- |         USA |                None |     88.0 |
- |         USA |            San Jose |     59.0 |
- +-------------+---------------------+----------+
-
-# aggregate by multiple keys (three)
-
->>> cast_cons(q, ['country','city','gender'], [], Avg('age'))
- +-------------+---------------------+--------+----------+
- |     country |                city | gender | Avg(age) |
- +-------------+---------------------+--------+----------+
- |     England |    Great Torrington |   male |    232.0 |
- |     England |  Maida Vale, London |   male |     97.0 |
- |     Finland |            Helsinki |   male |     40.0 |
- | Netherlands |           Rotterdam |   male |     79.0 |
- | New Zealand |            Auckland |   male |      N/A |
- |      Norway |                Oslo |   male |     83.0 |
- |      Sweden |           Stockholm |   male |    102.0 |
- | Switzerland |          Winterthur |   male |     75.0 |
- |         USA | Betty Jean Jennings | female |     85.0 |
- |         USA |             Chicago |   male |     60.0 |
- |         USA |           Milwaukee |   male |     72.0 |
- |         USA |            New York |   male |     56.0 |
- |         USA |                None | female |     88.0 |
- |         USA |            San Jose |   male |     59.0 |
- +-------------+---------------------+--------+----------+
+>>> cast_cons(q, ['born__country','born__city','gender'], [], Avg('age'))
+ +---------------+---------------------+--------+----------+
+ | born__country |          born__city | gender | Avg(age) |
+ +---------------+---------------------+--------+----------+
+ |       England |    Great Torrington |   male |    232.0 |
+ |       England |  Maida Vale, London |   male |     97.0 |
+ |       Finland |            Helsinki |   male |     40.0 |
+ |   Netherlands |           Amsterdam |   male |     49.0 |
+ |   Netherlands |           Rotterdam |   male |     79.0 |
+ |   New Zealand |            Auckland |   male |      N/A |
+ |        Norway |                Oslo |   male |     83.0 |
+ |  South Africa |            Pretoria |   None |     41.0 |
+ |        Sweden |           Stockholm |   male |    102.0 |
+ |   Switzerland |          Winterthur |   male |     75.0 |
+ |           USA |                None | female |     88.0 |
+ |           USA | Betty Jean Jennings | female |     85.0 |
+ |           USA |             Chicago | female |     60.0 |
+ |           USA |           Milwaukee |   male |     72.0 |
+ |           USA |            New York |   male |     56.0 |
+ |           USA |            San Jose |   male |     59.0 |
+ |           USA |             Seattle |   male |     54.0 |
+ +---------------+---------------------+--------+----------+
 
 # multiple aggregates -- NOT YET
 
-#>>> cast_cons(q, ['country','city'], [], [Avg('age'), Min('age'), Max('age')])
+#>>> cast_cons(q, ['born__country','born__city'], [], [Avg('age'), Min('age'), Max('age')])
 
 # city key not present, level empty, use higher level query
 
->>> cast_cons(people.find(nick='Kay'), ['name', 'country', 'city'])
- +--------------------+---------+------+------------+
- |               name | country | city | Count(all) |
- +--------------------+---------+------+------------+
- | Kathleen Antonelli |     USA | None |          1 |
- +--------------------+---------+------+------------+
+>>> cast_cons(people.find(nick='Kay'), ['name', 'born__country', 'born__city'])
+ +--------------------+---------------+------------+------------+
+ |               name | born__country | born__city | Count(all) |
+ +--------------------+---------------+------------+------------+
+ | Kathleen Antonelli |           USA |       None |          1 |
+ +--------------------+---------------+------------+------------+
 
 # city key not present, level empty, use higher level query
 
->>> cast_cons(q.find(nick='Kay'), ['city'])
- +------+------------+
- | city | Count(all) |
- +------+------------+
- | None |          1 |
- +------+------------+
+>>> cast_cons(q.find(nick='Kay'), ['born__city'])
+ +------------+------------+
+ | born__city | Count(all) |
+ +------------+------------+
+ |       None |          1 |
+ +------------+------------+
 
 # PIVOTING
 
 # pivoting by one factor
 
->>> cast_cons(q, ['country'], ['gender'])
- +-------------+--------+------+------------+
- |     country | female | male | Count(all) |
- +-------------+--------+------+------------+
- |     England |      0 |    2 |          2 |
- |     Finland |      0 |    1 |          1 |
- | Netherlands |      0 |    1 |          1 |
- | New Zealand |      0 |    1 |          1 |
- |      Norway |      0 |    1 |          1 |
- |      Sweden |      0 |    1 |          1 |
- | Switzerland |      0 |    1 |          1 |
- |         USA |      2 |    4 |          6 |
- +-------------+--------+------+------------+
+>>> cast_cons(q, ['born__country'], ['gender'])
+ +---------------+--------+------+------------+
+ | born__country | female | male | Count(all) |
+ +---------------+--------+------+------------+
+ |       England |      0 |    2 |          2 |
+ |       Finland |      0 |    1 |          1 |
+ |   Netherlands |      0 |    2 |          2 |
+ |   New Zealand |      0 |    1 |          2 |
+ |        Norway |      0 |    1 |          1 |
+ |  South Africa |      0 |    0 |          1 |
+ |        Sweden |      0 |    1 |          1 |
+ |   Switzerland |      0 |    1 |          1 |
+ |           USA |      3 |    4 |          7 |
+ +---------------+--------+------+------------+
 
 # pivoting by multiple factors, one level
 
->>> cast_cons(q.find(country='England'), ['country'], ['city', 'gender'])
- +---------+------------------+--------------------+------+------------+
- | country | Great Torrington | Maida Vale, London | male | Count(all) |
- +---------+------------------+--------------------+------+------------+
- | England |                1 |                  1 |    2 |          2 |
- +---------+------------------+--------------------+------+------------+
+>>> cast_cons(q.find(born__country='England'), ['born__country'], ['born__city', 'gender'])
+ +---------------+------------------+--------------------+------+------------+
+ | born__country | Great Torrington | Maida Vale, London | male | Count(all) |
+ +---------------+------------------+--------------------+------+------------+
+ |       England |                1 |                  1 |    2 |          2 |
+ +---------------+------------------+--------------------+------+------------+
 
 # pivoting by multiple factors, multiple levels
 
->>> cast_cons(q.find(country=not_('USA')), ['country'], ['city', 'gender'])
- +-------------+----------+------------------+----------+--------------------+------+-----------+-----------+------------+------+------------+
- |     country | Auckland | Great Torrington | Helsinki | Maida Vale, London | Oslo | Rotterdam | Stockholm | Winterthur | male | Count(all) |
- +-------------+----------+------------------+----------+--------------------+------+-----------+-----------+------------+------+------------+
- |     England |        0 |                1 |        0 |                  1 |    0 |         0 |         0 |          0 |    2 |          2 |
- |     Finland |        0 |                0 |        1 |                  0 |    0 |         0 |         0 |          0 |    1 |          1 |
- | Netherlands |        0 |                0 |        0 |                  0 |    0 |         1 |         0 |          0 |    1 |          1 |
- | New Zealand |        1 |                0 |        0 |                  0 |    0 |         0 |         0 |          0 |    1 |          1 |
- |      Norway |        0 |                0 |        0 |                  0 |    1 |         0 |         0 |          0 |    1 |          1 |
- |      Sweden |        0 |                0 |        0 |                  0 |    0 |         0 |         1 |          0 |    1 |          1 |
- | Switzerland |        0 |                0 |        0 |                  0 |    0 |         0 |         0 |          1 |    1 |          1 |
- +-------------+----------+------------------+----------+--------------------+------+-----------+-----------+------------+------+------------+
+>>> cast_cons(q.find(born__country__in=['Netherlands','Finland']), ['born__country'], ['born__city', 'gender'])
+ +---------------+-----------+----------+-----------+------+------------+
+ | born__country | Amsterdam | Helsinki | Rotterdam | male | Count(all) |
+ +---------------+-----------+----------+-----------+------+------------+
+ |       Finland |         0 |        1 |         0 |    1 |          1 |
+ |   Netherlands |         1 |        0 |         1 |    2 |          2 |
+ +---------------+-----------+----------+-----------+------+------------+
 
 # pivoting without grouper factors
 
->>> cast_cons(q, [], ['country'], Count())
- +---------+---------+-------------+-------------+--------+--------+-------------+-----+------------+
- | England | Finland | Netherlands | New Zealand | Norway | Sweden | Switzerland | USA | Count(all) |
- +---------+---------+-------------+-------------+--------+--------+-------------+-----+------------+
- |       2 |       1 |           1 |           1 |      1 |      1 |           1 |   6 |         14 |
- +---------+---------+-------------+-------------+--------+--------+-------------+-----+------------+
+>>> cast_cons(q, [], ['born__country'], Count())
+ +---------+---------+-------------+-------------+--------+--------------+--------+-------------+-----+------------+
+ | England | Finland | Netherlands | New Zealand | Norway | South Africa | Sweden | Switzerland | USA | Count(all) |
+ +---------+---------+-------------+-------------+--------+--------------+--------+-------------+-----+------------+
+ |       2 |       1 |           2 |           2 |      1 |            1 |      1 |           1 |   7 |         18 |
+ +---------+---------+-------------+-------------+--------+--------------+--------+-------------+-----+------------+
 
 # values are unwrapped (in this case some people had more than one occupation)
 
@@ -336,11 +297,17 @@ __doc__ = """
  +--------------------------------+--------+------+------------+
  |                     occupation | female | male | Count(all) |
  +--------------------------------+--------+------+------------+
+ |                 NetBSD founder |      0 |    0 |          1 |
+ |                OpenBSD founder |      0 |    0 |          1 |
+ |                OpenSSH founder |      0 |    0 |          1 |
  |           President of the FSF |      0 |    1 |          1 |
- |              R language author |      0 |    1 |          1 |
  |              Software engineer |      0 |    1 |          1 |
+ |        author of Perl language |      0 |    1 |          1 |
+ |      author of Python language |      0 |    1 |          1 |
+ |           author of R language |      0 |    1 |          2 |
  |              computer engineer |      0 |    1 |          1 |
  |      computer language pioneer |      0 |    1 |          1 |
+ |            computer programmer |      0 |    1 |          1 |
  |             computer scientist |      0 |    4 |          4 |
  |                   cryptanalyst |      0 |    1 |          1 |
  | invented mechanical calculator |      0 |    1 |          1 |
@@ -348,26 +315,28 @@ __doc__ = """
  |                  mathematician |      0 |    2 |          2 |
  |      original ENIAC programmer |      1 |    0 |          1 |
  |                     polititian |      0 |    1 |          1 |
- |                     programmer |      1 |    0 |          1 |
+ |                     programmer |      1 |    1 |          2 |
  +--------------------------------+--------+------+------------+
 
 # all row cells are present regardless of data availability in grouper columns
 
->>> cast_cons(q, ['country', 'nick'], [], Count('nick'))
- +-------------+-------+-------------+
- |     country |  nick | Count(nick) |
- +-------------+-------+-------------+
- |     England |  None |         N/A |
- |     Finland |  None |         N/A |
- | Netherlands |  None |         N/A |
- | New Zealand |  None |         N/A |
- |      Norway |  None |         N/A |
- |      Sweden | Conny |           1 |
- | Switzerland |  None |         N/A |
- |         USA |   Kay |           1 |
- |         USA |   Woz |           1 |
- |         USA |   rms |           1 |
- +-------------+-------+-------------+
+>>> cast_cons(q, ['born__country', 'nick'], [], Count('nick'))
+ +---------------+-------+-------------+
+ | born__country |  nick | Count(nick) |
+ +---------------+-------+-------------+
+ |       England |  None |         N/A |
+ |       Finland |  None |         N/A |
+ |   Netherlands |  None |         N/A |
+ |   New Zealand |  None |         N/A |
+ |        Norway |  None |         N/A |
+ |  South Africa |  None |         N/A |
+ |        Sweden | Conny |           1 |
+ |   Switzerland |  None |         N/A |
+ |           USA |   Kay |           1 |
+ |           USA |   Woz |           1 |
+ |           USA |   rms |           1 |
+ +---------------+-------+-------------+
+
 
 # summary function
 
@@ -375,13 +344,13 @@ __doc__ = """
  +-----+---------+--------+---------+---------+-----+
  | min | 1st qu. | median | average | 3rd qu. | max |
  +-----+---------+--------+---------+---------+-----+
- |  40 |      56 |     79 |    86.8 |    77.0 | 232 |
+ |  40 |    99.5 |   73.5 |    79.5 |    45.0 | 232 |
  +-----+---------+--------+---------+---------+-----+
 
 # standard deviation function
 
 >>> stdev(q, 'age')
-45.14421336118285
+42.176101401288442
 
 # nested structures
 
@@ -390,17 +359,22 @@ __doc__ = """
  | fullname__first | Count(all) |
  +-----------------+------------+
  |            Alan |          1 |
+ |           Anita |          1 |
  |          Donald |          1 |
+ |           Guido |          1 |
  |           Linus |          1 |
+ |         Richard |          1 |
  |         Stephen |          1 |
+ |            Theo |          1 |
  +-----------------+------------+
 
 >>> cast_cons(q, ['gender'], ['fullname__first'])
- +--------+------+--------+-------+---------+------------+
- | gender | Alan | Donald | Linus | Stephen | Count(all) |
- +--------+------+--------+-------+---------+------------+
- |   male |    1 |      1 |     1 |       1 |          4 |
- +--------+------+--------+-------+---------+------------+
+ +--------+------+-------+--------+-------+-------+---------+---------+------------+
+ | gender | Alan | Anita | Donald | Guido | Linus | Richard | Stephen | Count(all) |
+ +--------+------+-------+--------+-------+-------+---------+---------+------------+
+ | female |    0 |     1 |      0 |     0 |     0 |       0 |       0 |          3 |
+ |   male |    1 |     0 |      1 |     1 |     1 |       1 |       1 |         13 |
+ +--------+------+-------+--------+-------+-------+---------+---------+------------+
 
 """
 
@@ -435,7 +409,8 @@ class Level(object):
         self.children = []
     def attach(self, levels):
         "Attaches a depending factor level to this level."
-        for l in levels: assert(isinstance(l, Level))          # DEBUG
+        if __debug__:
+            for l in levels: assert(isinstance(l, Level))
         self.children.extend(levels)
     def get_rows(self):
         "Returns table rows. If more than one child exists, duplicate self for each of them."
@@ -458,28 +433,40 @@ class CatchAllLevel(object):
         self.query = query
     __unicode__ = __str__ = lambda self: '(all)'
 
-"""
-    group_by -- по каким ключам сгруппировать данные (ключ идет в заголовок столбца, значение -- в ячейку)
-    pivot_by -- по каким значениям дополнительно сгруппировать данные (это уже НЕ pipeline; значение идет в заголовок столбца)
-    agg(key) -- по какому ключу получать значение и какой функцией его обрабатывать (результат идет в ячейку pivot)
-
-    если сводные значения не указаны, делаем просто общий агрегат.
-
-    смысл "отливки" (cast) в формировании _табличной_ формы; перед этим делается группировка -- можно её код убрать в Query.
-"""
-
 def cast(basic_query, factor_names=None, pivot_factors=None, *aggregates):
     """
     Returns a table summarizing data grouped by given factors.
     Calculates aggregated values. If aggregate is not defined, Count() is used.
     Supports pivoting (i.e. using factor levels as columns).
+
+    The name "cast" stands for "casting melt data" and is a reference to Hadley
+    Wickham's package "reshape" for R language (http://had.co.nz/reshape/),
+    though internally these packages have little in common.
+
+    `basic_query` is a Query instance (pre-filtered or not) on which the table
+    is going to be built.
+
+    `factor_names` is an optional list of keys by which data will be grouped.
+    Their names will go into the table heading, and their values will be used
+    to calculate aggregated values. If more than one factor is specified, they
+    will be grouped hierarchically from left to right.
+
+    `pivot_factors` is an optional list of keys which values will go into the
+    table heading along with factor names so that extra columns with
+    aggregated values will be added for each possible factor level (key value).
+
+    `aggregates` is an optional list of Aggregate instances. Some aggregates
+    require a factor name (i.e. key). Examples: Count(), Sum('price').
+    Aggregates will be calculated for each combination of factors and for each
+    pivoted factor level. If aggregates are not specified, a Count() is added.
+
+    See tests for usage examples.
     """
-    # XXX TODO: multiple aggregates (they will also multiply pivot columns)
     
     # XXX this function actually *groups* data and creates a table.
     #     Move the grouping stuff to Query code as a method?
 
-    # note: do not declare the lists in func sig or they will migrate between calls :)
+    # note: mutables declared in func signature tend to migrate between calls ;)
     factor_names  = factor_names  or []
     pivot_factors = pivot_factors or []
     aggregates    = aggregates    or [Count()]
